'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DataLinkPacket = exports.DataLinkConnection = exports.MSEED_TYPE = exports.ENDSTREAM = exports.STREAM = exports.PACKET = exports.ERROR = exports.USER_BROWSER = exports.MAX_PROC_NUM = exports.STREAM_MODE = exports.QUERY_MODE = exports.DATALINK_PROTOCOL = exports.RSVP = exports.miniseed = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

/*global DataView */
/*global WebSocket */

/**
 * Philip Crotwell
 * University of South Carolina, 2016
 * http://www.seis.sc.edu
 */

var _util = require('./util');

var _seisplotjsMiniseed = require('seisplotjs-miniseed');

var miniseed = _interopRequireWildcard(_seisplotjsMiniseed);

var _rsvp = require('rsvp');

var RSVP = _interopRequireWildcard(_rsvp);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/* reexport */
exports.miniseed = miniseed;
exports.RSVP = RSVP;
var DATALINK_PROTOCOL = exports.DATALINK_PROTOCOL = "1.0";
var QUERY_MODE = exports.QUERY_MODE = "QUERY";
var STREAM_MODE = exports.STREAM_MODE = "STREAM";
var MAX_PROC_NUM = exports.MAX_PROC_NUM = Math.pow(2, 16) - 2;
var USER_BROWSER = exports.USER_BROWSER = "browser";

var ERROR = exports.ERROR = "ERROR";
var PACKET = exports.PACKET = "PACKET";
var STREAM = exports.STREAM = "STREAM";
var ENDSTREAM = exports.ENDSTREAM = "ENDSTREAM";
var MSEED_TYPE = exports.MSEED_TYPE = "MSEED";

var defaultHandleResponse = function defaultHandleResponse(message) {
  console.log("Unhandled datalink response: " + message);
};

var DataLinkConnection = exports.DataLinkConnection = function () {
  function DataLinkConnection(url, packetHandler, errorHandler) {
    _classCallCheck(this, DataLinkConnection);

    this.url = url;
    this.mode = QUERY_MODE;
    this.packetHandler = packetHandler;
    this.errorHandler = errorHandler;
    this.serverId = null;
    // meant to be processId, so use 1 <= num <= 2^15 to be safe
    this.clientIdNum = Math.floor(Math.random() * MAX_PROC_NUM) + 1;
    this.username = USER_BROWSER;
    this.responseResolve = null;
    this.responseReject = null;
  }

  /** creates the websocket connection and sends the client
  *  ID. Returns a Promise that resolves to the server's
  * ID.
  */


  _createClass(DataLinkConnection, [{
    key: 'connect',
    value: function connect() {
      var that = this;
      return new RSVP.Promise(function (resolve, reject) {
        var webSocket = new WebSocket(that.url, DATALINK_PROTOCOL);
        that.webSocket = webSocket;
        webSocket.binaryType = 'arraybuffer';
        webSocket.onmessage = function (event) {
          that.handle(event);
        };
        webSocket.onerror = function (event) {
          that.handleError(new Error("" + miniseed.model.stringify(event)));
          reject(event);
        };
        webSocket.onclose = function () {
          that.webSocket = null; // clean up
        };
        webSocket.onopen = function () {
          resolve(that);
        };
      }).then(function (datalink) {
        return datalink.sendId();
      }).then(function (idmsg) {
        that.serverId = idmsg;
        return idmsg;
      });
    }
  }, {
    key: 'stream',
    value: function stream() {
      if (this.mode === STREAM_MODE) {
        return;
      }
      this.mode = STREAM_MODE;
      this.sendDLCommand(STREAM, "");
    }
  }, {
    key: 'endStream',
    value: function endStream() {
      if (this.mode === QUERY_MODE) {
        return;
      }
      this.mode = QUERY_MODE;
      this.sendDLCommand(ENDSTREAM, "");
    }
  }, {
    key: 'close',
    value: function close() {
      if (this.webSocket) {
        this.endStream(); // end streaming just in case
        if (this.webSocket) {
          this.webSocket.close();
        }
        this.webSocket = null;
        this.mode = null;
      }
    }

    /**
    * Send a ID Command. Command is a string.
    * Returns a Promise.
    */

  }, {
    key: 'sendId',
    value: function sendId() {
      var that = this;
      return this.awaitDLCommand("ID seisplotjs:" + this.username + ":" + this.clientIdNum + ":javascript").then(function (replyMsg) {
        if (replyMsg.startsWith("ID DataLink ")) {
          that.serverId = replyMsg;
          return replyMsg;
        } else {
          throw new Error("not ID line: " + miniseed.model.stringify(replyMsg));
        }
      });
    }

    /** encodes command with optional data section as
     * a string. This works for client generated commands
     * but not for a PACKET, which would have binary data.
     * PACKET is what client receives, but should never
     * send as we do not generate data. */

  }, {
    key: 'encodeDLCommand',
    value: function encodeDLCommand(command, dataString) {
      var cmdLen = command.length;
      var len = 3 + command.length;
      var lenStr = "";
      if (dataString && dataString.length > 0) {
        lenStr = String(dataString.length);
        len += lenStr.length + 1;
        cmdLen += lenStr.length + 1;
        len += dataString.length;
      }
      var rawPacket = new ArrayBuffer(len);
      var packet = new DataView(rawPacket);
      packet.setUint8(0, 68); // ascii D
      packet.setUint8(1, 76); // ascii L
      packet.setUint8(2, cmdLen);
      var i = 3;
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = command[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var c = _step.value;

          packet.setUint8(i, c.charCodeAt(0));
          i++;
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      var SPACE = ' ';
      if (dataString && dataString.length > 0) {
        packet.setUint8(i, SPACE.charCodeAt(0)); // ascii space
        i++;
        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
          for (var _iterator2 = lenStr[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var _c = _step2.value;

            packet.setUint8(i, _c.charCodeAt(0));
            i++;
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
              _iterator2.return();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }

        var _iteratorNormalCompletion3 = true;
        var _didIteratorError3 = false;
        var _iteratorError3 = undefined;

        try {
          for (var _iterator3 = dataString[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
            var _c2 = _step3.value;

            packet.setUint8(i, _c2.charCodeAt(0));
            i++;
          }
        } catch (err) {
          _didIteratorError3 = true;
          _iteratorError3 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion3 && _iterator3.return) {
              _iterator3.return();
            }
          } finally {
            if (_didIteratorError3) {
              throw _iteratorError3;
            }
          }
        }
      }
      return rawPacket;
    }
  }, {
    key: 'sendDLCommand',
    value: function sendDLCommand(command, dataString) {
      console.log("send: " + command + " | " + (dataString ? dataString : ""));
      var rawPacket = this.encodeDLCommand(command, dataString);
      if (this.webSocket) {
        this.webSocket.send(rawPacket);
      } else {
        throw new Error("WebSocket has been closed.");
      }
    }

    /**
    * Send a DataLink Command and await the response. Command is a string.
    * Returns a Promise that resolves with the webSocket MessageEvent.
    */

  }, {
    key: 'awaitDLCommand',
    value: function awaitDLCommand(command, dataString) {
      var that = this;
      var promise = new RSVP.Promise(function (resolve, reject) {
        that.responseResolve = resolve;
        that.responseReject = reject;
        that.sendDLCommand(command, dataString);
      }).then(function (response) {
        that.responseResolve = null;
        that.responseReject = null;
        return response;
      }).catch(function (error) {
        that.responseResolve = null;
        that.responseReject = null;
        throw error;
      });
      return promise;
    }
  }, {
    key: 'handle',
    value: function handle(wsEvent) {
      var rawData = wsEvent.data;
      var dlPreHeader = new DataView(rawData, 0, 3);
      if ('D' === String.fromCharCode(dlPreHeader.getUint8(0)) && 'L' === String.fromCharCode(dlPreHeader.getUint8(1))) {
        var headerLen = dlPreHeader.getUint8(2);
        var header = (0, _util.dataViewToString)(new DataView(rawData, 3, headerLen));
        //console.log("handle wsEvent   header: '"+header+"'");
        if (header.startsWith(PACKET)) {
          if (this.packetHandler) {
            try {
              var _packet = new DataLinkPacket(header, new DataView(rawData, 3 + headerLen));
              this.packetHandler(_packet);
            } catch (e) {
              this.errorHandler(e);
            }
          } else {
            this.errorHandler(new Error("packetHandler not defined"));
          }
        } else if (header.startsWith(ERROR) || header.startsWith("OK")) {
          var split = header.split(' ');
          var value = split[1];
          // not needed as one datalink packet per web socket event
          // const dataSize = Number.parseInt(split[2]);
          var message = (0, _util.dataViewToString)(new DataView(rawData, 3 + headerLen));
          if (header.startsWith(ERROR)) {
            this.handleError(new Error("value=" + value + " " + message));
          } else if (header.startsWith("OK")) {
            if (this.responseResolve) {
              this.responseResolve(header + " | " + message);
              console.log(header + " | " + message);
            } else {
              console.log("OK without responseResolve");
            }
          }
        } else if (this.responseResolve) {
          this.responseResolve(header);
        } else {
          defaultHandleResponse(header);
        }
      } else {
        throw new Error("DataLink Packet did not start with DL");
      }
    }
  }, {
    key: 'handleError',
    value: function handleError(error) {
      if (this.responseReject) {
        this.responseReject(error);
      }
      if (this.errorHandler) {
        this.errorHandler(error);
      }
      console.log("handleError: " + error.message);
    }
  }]);

  return DataLinkConnection;
}();

var DataLinkPacket = exports.DataLinkPacket = function DataLinkPacket(header, dataview) {
  _classCallCheck(this, DataLinkPacket);

  this.header = header;
  this.data = dataview;
  var split = this.header.split(' ');
  this.streamId = split[1];
  this.pktid = split[2];
  this.hppackettime = split[3];
  this.hppacketstart = split[4];
  this.hppacketend = split[5];
  this.dataSize = Number.parseInt(split[6]);
  if (dataview.byteLength < this.dataSize) {
    throw new Error("not enough bytes in dataview for packet: " + this.dataSize);
  }
  if (this.streamId.endsWith(MSEED_TYPE)) {
    this.miniseed = miniseed.parseSingleDataRecord(dataview);
  } else {
    throw new Error("Unknown DataLink Packet type: " + this.streamId);
  }
};