'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.StreamStat = exports.RingserverConnection = exports.IRIS_HOST = exports.moment = exports.RSVP = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

/**
 * Philip Crotwell
 * University of South Carolina, 2017
 * http://www.seis.sc.edu
 */

exports.stationsFromStreams = stationsFromStreams;
exports.nslcSplit = nslcSplit;

var _rsvp = require('rsvp');

var _rsvp2 = _interopRequireDefault(_rsvp);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _seisplotjsMiniseed = require('seisplotjs-miniseed');

var miniseed = _interopRequireWildcard(_seisplotjsMiniseed);

var _util = require('./util');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

exports.RSVP = _rsvp2.default;
exports.moment = _moment2.default;
var IRIS_HOST = exports.IRIS_HOST = 'rtserve.iris.washington.edu';

var ORG = 'Organization: ';

var RingserverConnection = exports.RingserverConnection = function () {
  function RingserverConnection(host, port) {
    _classCallCheck(this, RingserverConnection);

    this._host = host ? host : IRIS_HOST;
    this._port = port ? port : 80;
  }

  _createClass(RingserverConnection, [{
    key: 'host',
    value: function host(value) {
      return (0, _util.hasArgs)(value) ? (this._host = value, this) : this._host;
    }
  }, {
    key: 'port',
    value: function port(value) {
      return (0, _util.hasArgs)(value) ? (this._port = value, this) : this._port;
    }

    /** Pulls id result from ringserver /id parsed into an object with
      * 'ringserverVersion' and 'serverId' fields.
      * Result returned is an RSVP Promise.
      */

  }, {
    key: 'pullId',
    value: function pullId() {
      return this.pullRaw(this.formIdURL()).then(function (raw) {
        var lines = raw.split('\n');
        var organization = lines[1];
        if (organization.startsWith(ORG)) {
          organization = organization.substring(ORG.length);
        }
        return {
          'ringserverVersion': lines[0],
          'serverId': organization
        };
      });
    }

    /**
     *  Use numeric level (1-6) to pull just IDs from ringserver.
     *  In a default ringserver,
     *  level=1 would return all networks like
     *  CO
     *  and level=2 would return all stations like
     *  CO_JSC
     *  If level is falsy/missing, level=6 is used.
     *  The optional matchPattern is a regular expression, so for example
     *  '.+_JSC_00_HH.' would get all HH? channels from any station name JSC.
     *  Result returned is an RSVP Promise.
     */

  }, {
    key: 'pullStreamIds',
    value: function pullStreamIds(level, matchPattern) {
      var queryParams = 'level=6';
      if (level && level > 0) {
        queryParams = 'level=' + level;
      }
      if (matchPattern) {
        queryParams = queryParams + '&match=' + matchPattern;
      }
      var url = this.formStreamIdsURL(queryParams);
      return this.pullRaw(url).then(function (raw) {
        return raw.split('\n').filter(function (line) {
          return line.length > 0;
        });
      });
    }

    /**
      * Pull streams, including start and end times, from the ringserver.
      * The optional matchPattern is a regular expression, so for example
      * '.+_JSC_00_HH.' would get all HH? channels from any station name JSC.
      * Result returned is an RSVP Promise.
      */

  }, {
    key: 'pullStreams',
    value: function pullStreams(matchPattern) {
      var queryParams = "";
      if (matchPattern) {
        queryParams = 'match=' + matchPattern;
      }
      var url = this.formStreamsURL(queryParams);
      return this.pullRaw(url).then(function (raw) {
        var lines = raw.split('\n');
        var out = {};
        out.accessTime = (0, _moment2.default)().utc();
        out.streams = [];
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = lines[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var line = _step.value;

            if (line.length === 0) {
              continue;
            }
            var vals = line.split(/\s+/);
            if (vals.length === 0) {
              // blank line, skip
              continue;
            } else if (vals.length >= 2) {
              out.streams.push(new StreamStat(vals[0], vals[1], vals[2]));
            } else {
              console.log("Bad /streams line, skipping: '" + line + "'");
            }
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }

        return out;
      });
    }

    /** Pulls raw result from ringserver /streams. QueryParams should
      * be formatted like URL query parameters, ie 'name=value&name=value'.
      * Result returned is an RSVP Promise.
      */

  }, {
    key: 'pullRaw',
    value: function pullRaw(url) {
      var promise = new _rsvp2.default.Promise(function (resolve, reject) {
        var client = new XMLHttpRequest();
        client.open("GET", url);
        client.onreadystatechange = handler;
        client.responseType = "text";
        client.setRequestHeader("Accept", "text/plain");
        client.send();

        function handler() {
          if (this.readyState === this.DONE) {
            if (this.status === 200) {
              resolve(this.response);
            } else {
              console.log("Reject pullRaw: " + url + " " + this.status);
              reject(this);
            }
          }
        }
      });
      return promise;
    }
  }, {
    key: 'formBaseURL',
    value: function formBaseURL() {
      return 'http://' + this._host + (this._port == 80 ? '' : ':' + this._port);
    }
  }, {
    key: 'formIdURL',
    value: function formIdURL() {
      return this.formBaseURL() + '/id';
    }
  }, {
    key: 'formStreamsURL',
    value: function formStreamsURL(queryParams) {
      return this.formBaseURL() + '/streams' + (queryParams && queryParams.length > 0 ? '?' + queryParams : '');
    }
  }, {
    key: 'formStreamIdsURL',
    value: function formStreamIdsURL(queryParams) {
      return this.formBaseURL() + '/streamids' + (queryParams && queryParams.length > 0 ? '?' + queryParams : '');
    }
  }]);

  return RingserverConnection;
}();

function stationsFromStreams(streams) {
  var out = new Map();
  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = streams[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      var s = _step2.value;

      var nslc = nslcSplit(s.key);
      var staKey = nslc.networkCode + "." + nslc.stationCode;
      var stat = out.get(staKey);
      if (!(0, _util.isDef)(stat)) {
        stat = new StreamStat(staKey, s.startRaw, s.endRaw);
        out.set(staKey, stat);
      } else {
        if (stat.start.isAfter(s.start)) {
          stat.start = s.start;
          stat.startRaw = s.startRaw;
        }
        if (stat.end.isBefore(s.end)) {
          stat.end = s.end;
          stat.endRaw = s.endRaw;
        }
      }
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2.return) {
        _iterator2.return();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }

  return Array.from(out.values());
}

function nslcSplit(id) {
  var split = id.split('/');
  var out = {};
  out.type = split[1];
  var nslc = split[0].split('_');
  if (nslc.length === 4) {
    // assume net, station, loc, chan
    out.networkCode = nslc[0];
    out.stationCode = nslc[1];
    out.locationCode = nslc[2];
    out.channelCode = nslc[3];
  } else {
    throw new Error("tried to split, did not find 4 elements in array: " + id);
  }
  return out;
}

var StreamStat = exports.StreamStat = function () {
  function StreamStat(key, start, end) {
    _classCallCheck(this, StreamStat);

    this.key = key;
    this.startRaw = start;
    this.endRaw = end;
    if (this.startRaw.indexOf('.') != -1 && this.startRaw.indexOf('.') < this.startRaw.length - 4) {
      this.startRaw = this.startRaw.substring(0, this.startRaw.indexOf('.') + 4);
    }
    if (this.startRaw.charAt(this.startRaw.length - 1) != 'Z') {
      this.startRaw = this.startRaw + 'Z';
    }
    if (this.endRaw.indexOf('.') != -1 && this.endRaw.indexOf('.') < this.endRaw.length - 4) {
      this.endRaw = this.endRaw.substring(0, this.endRaw.indexOf('.') + 4);
    }
    if (this.endRaw.charAt(this.endRaw.length - 1) != 'Z') {
      this.endRaw = this.endRaw + 'Z';
    }
    this.start = _moment2.default.utc(this.startRaw);
    this.end = _moment2.default.utc(this.endRaw);
    this.startRaw = start; // reset to unchanged strings
    this.endRaw = end;
  }

  _createClass(StreamStat, [{
    key: 'calcLatency',
    value: function calcLatency(accessTime) {
      return this.end.from(accessTime);
    }
  }]);

  return StreamStat;
}();